#!/usr/bin/env python
# -*- coding:utf-8 -*-

# Program: 分析tpcc-mysql的日志生成csv和图片
# Author : HH
# Date   : 2016-02-18

import sys
import os
import linecache
import xlwt
import time
import shutil
import re

reload(sys)
sys.setdefaultencoding('utf-8')

class TPCCReport(object):
  '''
  通过分析生成excel报表或图片
  '''
  def __init__(self, file_name):
    '''
    初始化参数
    '''
    self.file_name = file_name
    # 创建保存文件目录
    self.tpcc_dir = 'tpcc_{time}'.format(
      time = time.strftime('%Y-%m-%d_%H-%M-%S')
    )
    os.mkdir(self.tpcc_dir)

  def copy_log_file(self):
    '''
    复制需要分析的文件
    '''
    new_file_name = '{tpcc_dir}/{file_name}'.format(
      tpcc_dir  = self.tpcc_dir,
      file_name = os.path.basename(self.file_name)
    )
    shutil.copyfile(self.file_name, new_file_name)

  def get_split_value(self, data_list, sep='|'):
    '''
    通过给定的数据列表，和指定的分隔符分割数据，并返回分割后的结果
    '''
    return [re.split(sep, data.strip()) for data in data_list]

  def get_item_value(self, data_list, sep='|'):
    '''
    获得模块值的指标值
    '''
    return [[re.split(sep, col.strip()) for col in row] for row in data_list]

  def split_list(self, data_list, start_index, end_index=None):
    '''
    切割数组获得需要的列表值
    '''
    return [data[start_index:end_index] for data in data_list]
  
  def build_data_list(self, data_list, module_name_list=[], 
                      item_name_list=[]):
    '''
    获得数据数据列表。如:{'module':'count', 
                          'items' :[new_order, payment],
                          'values':[ [1,2], [1,2] ]}
    '''
    data_dict_list = []
    # 创建模块和指标的基本信息
    for module_name in module_name_list:
      data_dict_base = {
        'module': module_name,
        'items' : item_name_list,
        'values': []
      }
      data_dict_list.append(data_dict_base)

    # 像每一个模块中添加数据
    for row_num, row in enumerate(data_list, 0):
      for module_index in xrange(len(module_name_list)):
        data_dict_list[module_index]['values'].append([])
      for col_num, col in enumerate(row, 0):
        for item_index, item in enumerate(col, 0):
          data_dict_list[item_index]['values'][row_num].append(item)
    return data_dict_list

  def get_effect_line_list(self, start_tag='', end_tag='', file_name=None):
    '''
    过滤orzdba我们不需要的信息
    '''
    if file_name:
      self.file_name = file_name
    line_list = linecache.getlines(self.file_name)
    start_num = 0
    end_num = 0
    # 定义获取空白行
    space_line_list = []
    # 获取有效范围
    for num, line in enumerate(line_list, 0):
      if start_tag in line:
        start_num = num + 1
      if end_tag in line:
        end_num = num
        break
      # 获取空白行
      if start_num <> 0:
        tmp_line = line.strip()
        if not tmp_line:
          space_line_list.append(line)
    # 获取有效行
    effect_line_list = line_list[start_num:end_num]
    # 过滤空白行
    for line in space_line_list:
      effect_line_list.remove(line)
    return effect_line_list

  def create_picture(self, index_value, data_dict_list):
    '''
    创建图像
    '''
    # 导入数据分析和绘图包
    import matplotlib
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import numpy as np
    import pandas as pd

    for dict_index, data_dict in enumerate(data_dict_list, 0):
      # 以items为列名
      columns = data_dict['items']
      # 通过收集的数据和列名生成DataFrame对象
      df = pd.DataFrame(data_dict['values'],
                        index=index_value,
                        columns=columns)
      # 将数据转化成数字
      df = df.apply(pd.to_numeric, args=('coerce',))
      # 获得x坐标每个坐标展示的长度,以20的长度计算
      row_count = len(df.index)
      offset = row_count / 30 + 1
      # 画图
      df.plot(title   = data_dict['module'],
              figsize = (20, 10),
              xticks  = range(0, row_count, offset),
              grid    = True)
      # 设置x轴字倾斜
      locs, labels = plt.xticks()
      plt.setp(labels, rotation=60)
      # 保存图片
      plt.savefig(r'{pic_dir}/{file_name}.jpg'.format(
        pic_dir = self.tpcc_dir,
        file_name=data_dict['module']
      ))

      # 保存为csv
      df.to_csv(r'{pic_dir}/{file_name}.csv'.format(
        pic_dir = self.tpcc_dir,
        file_name=data_dict['module']
      ))
    
if __name__ == '__main__':
  # 获取分析文件名
  if len(sys.argv) <> 2:
    print 'Usage: python orzdba_report.py logFileName'
    os._exit(1)
  file_name = sys.argv[1]
  tpcc_report = TPCCReport(file_name)
  tpcc_report.copy_log_file()
  
  # 获取有效的收集的数据信息
  effect_line_list = tpcc_report.get_effect_line_list(
    start_tag = 'MEASURING START.',
    end_tag   = 'STOPPING THREADS'
  )
  # 定义模块名
  module_name_list = [
    'test_count',
    'late_count',
    '90%respons_time',
    'max_respons_time'
  ]
  # 指标名
  item_name_list = [
    'new_order',
    'payment',
    'order_status',
    'delivery',
    'stock_level'
  ]
  # 获得每个模块的值
  module_value = tpcc_report.get_split_value(
    effect_line_list, 
    sep = '\s*,\s*'
  )
  # 获得索引数组
  index_value = tpcc_report.split_list(module_value, None, 1)
  index_value = [value[0] for value in index_value]
  # 获得展示的数据
  module_value = tpcc_report.split_list(module_value, 1, None)
  # 通过模块值获得每个指标值
  item_value = tpcc_report.get_item_value(
    module_value,
    sep = '\(|\):|\||\s*'
  )
  # 获取格式化后的模块、指标名、指标值
  data_dict_list = tpcc_report.build_data_list(
    item_value,
    module_name_list,
    item_name_list
  )
  # 生成报表图片
  tpcc_report.create_picture(index_value, data_dict_list)
